--

DynamicMountFix = {};

function DynamicMountFix.prerequisitesPresent(specializations)
    return true
end;

function DynamicMountFix:loadMap(name)
	if g_server ~= nil then
		print("DynamicMountFix aktiviert auf Server");
		DynamicMountAttacher.updateTick = Utils.overwrittenFunction(DynamicMountAttacher.updateTick, DynamicMountFix.updateTickS);
		DynamicMountAttacher.onDetach = Utils.overwrittenFunction(DynamicMountAttacher.onDetach, DynamicMountFix.onDetachS);
		DynamicMountAttacher.update = Utils.overwrittenFunction(DynamicMountAttacher.update, DynamicMountFix.updateS);
		MountableObject.mountDynamic = Utils.overwrittenFunction(MountableObject.mountDynamic, DynamicMountFix.mountDynamic);
		MountableObject.dynamicMountTriggerCallback = DynamicMountFix.dynamicMountTriggerCallback;
	else
		print("DynamicMountFix deaktiviert auf Client");
	end;
end;


function DynamicMountFix:deleteMap()
	if self.dynamicMountTriggerId ~= nil then
		removeTrigger(self.dynamicMountTriggerId);
	end;
end;

function DynamicMountFix:mouseEvent(posX, posY, isDown, isUp, button)
end;

function DynamicMountFix:keyEvent(unicode, sym, modifier, isDown)
end;

function DynamicMountFix:update(dt)
end;

function DynamicMountFix:updateS(dt)
	isReleased = false;
	if not InputBinding.isPressed(InputBinding.DYNAMICMOUNTFIX) and self.lastIsPressed then
		isReleased = true;
	end;
	if isReleased then
		for k,vehicle in pairs(g_currentMission.steerables) do
			if vehicle:getIsActive() then
				local px, py, pz = getWorldTranslation(vehicle.rootNode);
				local nearestDistance = 20.0;
				local vx, vy, vz = getWorldTranslation(self.rootNode);
				local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
				if distance < nearestDistance and not self:getIsActive() and self.attacherVehicle ~= nil then
					for object,_ in pairs(self.pendingDynamicMountObjects) do
						if self.dynamicMountedObjects[object] == nil then
							if object:mountDynamic(self, self.dynamicMountAttacherTrigger.rootNode, self.dynamicMountAttacherTrigger.jointNode, true, 999999999999999999) then
								if object.nodeId ~= nil then
									self:addDynamicMountedObject(object);
								else
									self:addDynamicMountedObject(object);
								end;
							end;
						end;
					end;
				end;
				break;
			end;
		end;
	end;
	self.lastIsPressed = InputBinding.isPressed(InputBinding.DYNAMICMOUNTFIX);
end;

function DynamicMountFix:updateTickS(dt)

end;

function DynamicMountFix:updateTick(dt)
end;
function DynamicMountFix:draw()
end;
function DynamicMountFix:onLeave()
end;
function DynamicMountFix:onDetachS()
	for object,_ in pairs(self.dynamicMountedObjects) do
		object:unmountDynamic();
		if object.nodeId ~= nil then
			self:removeDynamicMountedObject(object, false);
		else
			self:removeDynamicMountedObject(object, false);
		end;
	end;
end;




function DynamicMountFix:mountDynamic(func, object, objectActorId, jointNode, singleAxisMovement, forceLimitScale)
	if not self:getSupportsMountDynamic() or self.mountObject ~= nil or self.dynamicMountObject ~= nil then
		return false;
	end	
	
	local constr = JointConstructor:new();
	constr:setActors(objectActorId, self.nodeId);
	constr:setJointTransforms(jointNode, jointNode);

	constr:setRotationLimit(0, 0, 0);
	constr:setRotationLimit(1, 0, 0);
	constr:setRotationLimit(2, 0, 0);
	if singleAxisMovement then
		constr:setTranslationLimit(0, true, -0.005, 0.005);
		constr:setTranslationLimit(1, true, -0.005, 0.005);
		constr:setTranslationLimit(2, true, -0.005, 0.005);
		constr:setEnableCollision(false);
	else
		constr:setTranslationLimit(0, false, 0, 0); 
		constr:setTranslationLimit(1, false, 0, 0);
		constr:setTranslationLimit(2, false, 0, 0);
		constr:setLinearDrive(0, false, true, 0, 0, 999999999999999, 0, 0);
		constr:setLinearDrive(1, false, true, 0, 0, 999999999999999, 0, -1);
		constr:setLinearDrive(2, false, true, 0, 0, 999999999999999, 0, 0);
		constr:setEnableCollision(true);
	end

	self.dynamicMountJointIndex = constr:finalize();
	self.dynamicMountObjectActorId = objectActorId;
	self.dynamicMountObject = object;
	self.dynamicMountObject:addDynamicMountedObject(self);

	return true;
end

local oldMountableObjectUpdate = MountableObject.update;
function MountableObject:update(dt)
	oldMountableObjectUpdate(self,dt);
	actionDone = false;
	if InputBinding.hasEvent(InputBinding.DYNAMICMOUNTFIX) and self.objToMount ~= nil and not self.isObjMounted then
		self:mountDynamic(self.objToMount, self.objToMount.dynamicMountAttacherNode, self.objToMount.dynamicMountAttacherNode, DynamicMountUtil.TYPE_FORK, self.dynamicMountTriggerForceAcceleration*self.objToMount.dynamicMountAttacherForceLimitScale);
		self.dynamicMountObjectTriggerCount = 1;
		self.isObjMounted = true;
		actionDone = true;
	end;
	if InputBinding.hasEvent(InputBinding.DYNAMICMOUNTFIX) and self.isObjMounted and not actionDone then
		self:unmountDynamic();
		self.dynamicMountObjectTriggerCount = nil;
		self.isObjMounted = false;
	end;
end;

function DynamicMountFix:dynamicMountTriggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
	if onEnter then
		if self.mountObject == nil then
			local vehicle = g_currentMission.nodeToVehicle[otherActorId];
			if vehicle ~= nil and vehicle.dynamicMountAttacherNode ~= nil then
				if self.dynamicMountObjectActorId == nil then
					self.objToMount = vehicle;
				elseif otherActorId ~= self.dynamicMountObjectActorId and self.dynamicMountObjectTriggerCount == nil then
					self.objToMount = vehicle;
				elseif otherActorId == self.dynamicMountObjectActorId then
					if self.dynamicMountObjectTriggerCount ~= nil then
						self.dynamicMountObjectTriggerCount = self.dynamicMountObjectTriggerCount+1;
					end
				end
			end
		end
	elseif onLeave then
		local vehicle = g_currentMission.nodeToVehicle[otherActorId];
		if vehicle ~= nil and vehicle.dynamicMountAttacherNode ~= nil then
			self.objToMount = nil;
		end;
		if otherActorId == self.dynamicMountObjectActorId and self.dynamicMountObjectTriggerCount ~= nil then
			self.dynamicMountObjectTriggerCount = self.dynamicMountObjectTriggerCount-1;
			if self.dynamicMountObjectTriggerCount == 0 then
				self:unmountDynamic();
				self.objToMount = nil;
				self.dynamicMountObjectTriggerCount = nil;
			end
		end
	end
end;

addModEventListener(DynamicMountFix);